@@ -6,6 +6,7 @@ from .fields import (PrimaryKeyField, EmailField, CharField,
                      ForeignKeyField, AmountField, OneToOneField)
 
 from .utils import Choices
+from .compat import python_2_unicode_compatible
 from .query import InsertQuery, UpdateQuery
 
 
@@ -16,6 +17,7 @@ class BaseModel(BaseApiModel):
     update_date = DateTimeField(api_name='UpdateDate')
 
 
+@python_2_unicode_compatible
 class User(BaseModel):
     TYPE_CHOICES = Choices(
         ('NATURAL_PERSON', 'natural', 'Natural person'),
@@ -39,8 +41,8 @@ class User(BaseModel):
         verbose_name = 'user'
         verbose_name_plural = 'users'
 
-    def __unicode__(self):
-        return u'%s %s' % (self.first_name, self.last_name)
+    def __str__(self):
+        return '%s %s' % (self.first_name, self.last_name)
 
 
 class Beneficiary(BaseModel):
@@ -77,6 +79,7 @@ class StrongAuthentication(BaseModel):
         }
 
 
+@python_2_unicode_compatible
 class Wallet(BaseModel):
     name = CharField(api_name='Name', required=True)
 
@@ -97,14 +100,14 @@ class Wallet(BaseModel):
         verbose_name = 'wallet'
         verbose_name_plural = 'wallets'
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
 
 class PaymentCard(BaseApiModel):
     id = PrimaryKeyField(api_name='ID')
     tag = CharField(api_name='Tag', required=True)
-    owner = ForeignKeyField(User, api_name='UserID', required=True,
+    owner = ForeignKeyField(User, api_name='OwnerID', required=True,
                             related_name='payment_cards')
     card_number = CharField(api_name='CardNumber', required=True)
     redirect_url = CharField(api_name='RedirectURL')
@@ -188,6 +191,8 @@ class Contribution(BaseModel):
     payment_card = ForeignKeyField(PaymentCard, api_name='PaymentCardID')
     type = CharField(api_name='Type')  # type of transaction: payline, ogone
     culture = CharField(api_name='Culture')
+    answer_code = CharField(api_name='AnswerCode')
+    answer_message = CharField(api_name='AnswerMessage')
 
     class Meta:
         verbose_name = 'contribution'
@@ -197,6 +202,59 @@ class Contribution(BaseModel):
         return self.is_succeeded and self.is_completed
 
 
+class ImmediateContribution(BaseModel):
+    user = ForeignKeyField(User, api_name='UserID', required=True,
+                           related_name='immediate_contributions')
+    wallet = ForeignKeyField(Wallet, api_name='WalletID', required=True,
+                             related_name='immediate_contributions')
+    amount = AmountField(api_name='Amount', required=True)
+    client_fee_amount = AmountField(api_name='ClientFeeAmount')
+    leetchi_fee_amount = AmountField(api_name='LeetchiFeeAmount')
+    is_succeeded = BooleanField(api_name='IsSucceeded')
+    is_completed = BooleanField(api_name='IsCompleted')
+    error = CharField(api_name='Error')
+    payment_card = ForeignKeyField(PaymentCard, api_name='PaymentCardID')
+    type = CharField(api_name='Type')
+    answer_code = CharField(api_name='AnswerCode')
+    answer_message = CharField(api_name='AnswerMessage')
+    culture = CharField(api_name='Culture')
+
+    class Meta:
+        verbose_name = 'immediate-contribution'
+        verbose_name_plural = 'immediate-contributions'
+
+    def is_success(self):
+        return self.is_succeeded and self.is_completed
+
+
+class RecurrentContribution(BaseModel):
+    user = ForeignKeyField(User, api_name='UserID', required=True,
+                           related_name='recurrent_contributions')
+    wallet = ForeignKeyField(Wallet, api_name='WalletID', required=True,
+                             related_name='recurrent_contributions')
+    amount = AmountField(api_name='Amount', required=True)
+    client_fee_amount = AmountField(api_name='ClientFeeAmount')
+    leetchi_fee_amount = AmountField(api_name='LeetchiFeeAmount')
+    is_succeeded = BooleanField(api_name='IsSucceeded')
+    is_completed = BooleanField(api_name='IsCompleted')
+    is_enabled = BooleanField(api_name='IsEnabled')
+    payment_url = CharField(api_name='PaymentURL')
+    template_url = CharField(api_name='TemplateURL')
+    return_url = CharField(api_name='ReturnURL', required=True)
+    error = CharField(api_name='Error')
+    culture = CharField(api_name='Culture')
+    start_date = DateTimeField(api_name='StartDate', required=True)
+    frequency_code = CharField(api_name='FrequencyCode', required=True)
+    number_of_executions = IntegerField(api_name='NumberOfExecutions', required=True)
+
+    class Meta:
+        verbose_name = 'recurrent-contribution'
+        verbose_name_plural = 'recurrent-contributions'
+
+    def is_success(self):
+        return self.is_succeeded and self.is_completed
+
+
 class Withdrawal(BaseModel):
     wallet = ForeignKeyField(Wallet, api_name='WalletID', required=True,
                              related_name='withdrawals')
@@ -244,6 +302,7 @@ class Refund(BaseModel):
         return self.is_succeeded and self.is_completed
 
 
+@python_2_unicode_compatible
 class Operation(BaseModel):
     user = ForeignKeyField(User, api_name='UserID', required=True,
                            related_name='operations')
@@ -257,3 +316,8 @@ class Operation(BaseModel):
     class Meta:
         verbose_name = 'operation'
         verbose_name_plural = 'operations'
+
+    def __str__(self):
+        return '<Operation n.%s: %s> %d' % (self.transaction_id,
+                                            self.transaction_type,
+                                            self.amount)
